#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <netinet/in.h>
#include <stdlib.h>
#include "libpq-fe.h"
#include "dbi.h"
#include "string-enum.h"

extern char *log_filename;

int SetupDB(const char *dbhost, struct SetupParameter *setupParameter)
{
    log_filename = "/usr/local/lamost/var/log/ccdendlog";
    char conninfo[MAXLINE]= "dbname=setups host= ";
    PGconn     *conn;
    int 	ret=1;

    strcat(conninfo,dbhost);
    conn = PQconnectdb(conninfo);
    if (PQstatus(conn) != CONNECTION_OK)
    {
	flogerror("Connection to database failed: %s", PQerrorMessage(conn));
	PQfinish(conn);
	return -1;
    }

    if(setupParameter->exposure>=0){
	char exposure[16];
	sprintf(exposure,"%d",setupParameter->exposure);
	flogerror("Setting exposure to %s",exposure);
	ret &= sqlset(conn,"exposure","current","value",(const char *)exposure);
    }

    if(setupParameter->nr>=0){
	char nr[16];
	sprintf(nr,"%d",setupParameter->nr);
	flogerror("Setting nr to %s",nr);
	ret &= sqlset(conn,"nr","current","value",nr);
    }

    if(setupParameter->sc>=0){
	char sc[16];
	sprintf(sc,"%d",setupParameter->sc);
	flogerror("Setting sc to %s",sc);
	ret &= sqlset(conn,"sc","current","value",sc);
    }

    if(setupParameter->nc>=0){
	char nc[16];
	sprintf(nc,"%d",setupParameter->nc);
	flogerror("Setting nc to %s",nc);
	ret &= sqlset(conn,"nc","current","value",nc);
    }

    if(setupParameter->sr>=0){
	char sr[16];
	sprintf(sr,"%d",setupParameter->sr);
	flogerror("Setting sr to %s",sr);
	ret &= sqlset(conn,"sr","current","value",sr);
    }

    if((int)setupParameter->binning>=0){
	flogerror("Setting binning to %s",binning2string(setupParameter->binning));
	ret &= sqlset(conn,"binning","current","value",(const char*)binning2string(setupParameter->binning));
    }
    if((int)setupParameter->gain>=0){
	flogerror("Setting gain to %s",gain2string(setupParameter->gain));
	ret &= sqlset(conn,"gain","current","value",(const char*)gain2string(setupParameter->gain));
    }
    if((int)setupParameter->obstype>=0){
	flogerror("Setting obstype to %s",obstype2string(setupParameter->obstype));
	ret &= sqlset(conn,"obstype","current","value",(const char*)obstype2string(setupParameter->obstype));
    }
    if((int)setupParameter->readspeed>=0){
	flogerror("Setting readspeed to %s",readspeed2string(setupParameter->readspeed));
	ret &= sqlset(conn,"readspeed","current","value",(const char*)readspeed2string(setupParameter->readspeed));
    }
    if((int)setupParameter->record>=0){
	flogerror("setting Recording to %s",record2string(setupParameter->record));
	ret &= sqlset(conn,"record","current","value",(const char*)record2string(setupParameter->record));
    }

    if(strcmp(setupParameter->record_dir,"")!=0){
	flogerror("Setting record_dir to %s",setupParameter->record_dir);
	ret &= sqlset(conn,"record_dir","current","value",setupParameter->record_dir);
    }
    if(strcmp(setupParameter->record_root,"")!=0){
	flogerror("Setting record_root to %s",setupParameter->record_root);
	ret &= sqlset(conn,"record_root","current","value",setupParameter->record_root);
    }
    if(strcmp(setupParameter->record_suffix,"")!=0){
	flogerror("Setting record_suffix to %s",setupParameter->record_suffix);
	ret &= sqlset(conn,"record_suffix","current","value",setupParameter->record_suffix);
    }
    PQfinish(conn);
    //  "observername" item is under telefocus database, so we connect to that
    sprintf(conninfo,"dbname=telefocus host= ");
    strcat(conninfo,dbhost);
    conn = PQconnectdb(conninfo);
    if (PQstatus(conn) != CONNECTION_OK)
    {
	flogerror("Connection to database failed: %s",
		PQerrorMessage(conn));
	PQfinish(conn);
	return -1;
    }

    if(strcmp(setupParameter->observername,"")!=0){
	flogerror("Setting observername to %s",setupParameter->observername);
	ret &= sqlset(conn,"observer","lamost","observername",setupParameter->observername);
    }

    if(ret==0){
	PQfinish(conn);
	return -1;
    }
    PQfinish(conn);

    return 0;
} 


//	set the database value.
//	returns boolean value: 0=failure 1=success
int sqlset(PGconn *conn,const char *table,const char *row,const char *col,const char *value){
    PGresult   *res;
    char 	sql[MAXLINE];
    sprintf(sql,"update %s set %s='%s' where name0x3b2d29='%s'",table,col,value,row);
    res = PQexec(conn,sql);
    if(PQresultStatus(res)!=PGRES_COMMAND_OK){
	flogerror("sqlset failed: %s", PQerrorMessage(conn));
	PQclear(res);
	return 0;
    }
    PQclear(res);
    return 1;
}
int SetDBValue(const char *host,const char *db,const char *table,const char *row,const char *column,const char *value){
    log_filename = "/usr/local/lamost/var/log/ccdendlog";
    char conninfo[MAXLINE];
    PGconn     *conn;
    int 	ret=1;

    sprintf(conninfo,"dbname=%s host=%s",db,host);
    conn = PQconnectdb(conninfo);
    if (PQstatus(conn) != CONNECTION_OK)
    {
	flogerror("Connection to database failed: %s", PQerrorMessage(conn));
	PQfinish(conn);
	return -1;
    }

    sqlset(conn,table,row,column,value);
    PQfinish(conn);
}
int sqlget_s(PGconn *conn,const char *table,const char *row,const char *col,char *value,int n){
    PGresult   *res;
    char 	sql[MAXLINE];
    sprintf(sql,"select %s from %s where name0x3b2d29='%s'",col,table,row);
    res = PQexec(conn,sql);
    if(PQresultStatus(res)!=PGRES_TUPLES_OK){
	flogerror("sqlget_s failed: %s", PQerrorMessage(conn));
	PQclear(res);
	return 0;
    }
    if(PQntuples(res)>0){
	snprintf(value,n,"%s",PQgetvalue(res,0,0));
    }
    PQclear(res);
    return 1;
}
int GetDBValue(const char *host,const char *db,const char *table,const char *row,const char *column,char *value,int n){
    log_filename = "/usr/local/lamost/var/log/ccdendlog";
    char conninfo[MAXLINE];
    PGconn     *conn;
    int 	ret=1;

    sprintf(conninfo,"dbname=%s host=%s",db,host);
    conn = PQconnectdb(conninfo);
    if (PQstatus(conn) != CONNECTION_OK)
    {
	flogerror("Connection to database failed: %s", PQerrorMessage(conn));
	PQfinish(conn);
	return -1;
    }

    sqlget_s(conn,table,row,column,value,n);
    PQfinish(conn);
}
//@procedure: svconnect - A connection will be open to the server.
// @param: address - notifier host address
// @param: port - port number
// @param: id - register id in notifier
// @return: a connection filedescriptor, or -1 on connection failed
int svconnect(const char *address, int port, const char *id) 
{
    struct sockaddr_in sin;
    int sockfd,ret;
    char command[128] = "id ";
    strcat(command,id);
    strcat(command,"\n");

    sin.sin_family = AF_INET;
    sin.sin_port = htons(port);
    sin.sin_addr.s_addr = inet_addr(address);
    sockfd = socket(PF_INET,SOCK_STREAM,0);
    if(sockfd < 0) {
	return(-1);
    }
    //flogerror("connecting to %s %s\n",address,port);
    ret = connect(sockfd,(struct sockaddr *)&sin,sizeof(sin));
    if(ret < 0) {
	return -1;
    }
    //flogerror("sockfd is %d\n",sockfd);
    if(write(sockfd,command,strlen(command)) < 0 ){
	return -1;
    }

    return sockfd;

}
